<?php

use Core\ConfigHandler;
use Core\Date\DateTime;
use Core\Widget;
use Core\ModelFactory;


class CronWidget extends Widget {
	/**
	 * The view for the admin dashboard.  Gets the last executed crons and displays that to the admin.
	 */
	public function dashboard() {
		// This dashboard has no effect if the user can't view crons.
		if(!\Core\user()->checkAccess('g:admin')){
			return '';
		}

		$view = $this->getView();

		$cronsEnabled = ConfigHandler::Get('/cron/enabled');
		$cronKeys = [
			'1-minute',
			'5-minute',
			'15-minute',
			'30-minute',
			'hourly',
			'2-hour',
			'3-hour',
			'6-hour',
			'12-hour',
			'daily',
			'weekly',
			'monthly',
		];
		$crons = [];
		$latestCron = 0;

		foreach($cronKeys as $cron) {
			$lastRan = ConfigHandler::Get('/cron/last/' . $cron);

			$crons[ $cron ] = $lastRan;
			$latestCron = max($latestCron, $lastRan);
		}

		$runTime = ConfigHandler::Get('/cron/run/daily');
		if($runTime === -1) {
			$dailySchedule = 'Any time';
		}
		else {
			$dailySchedule = 'During hour ' . $runTime;
		}

		$runTime = ConfigHandler::Get('/cron/run/daily');
		if($runTime === -1) {
			$dailySchedule = 'Any time';
		}
		else {
			$dailySchedule = 'During hour ' . $runTime;
		}

		$runTime = ConfigHandler::Get('/cron/run/daily');
		if($runTime === -1) {
			$dailySchedule = 'Any time';
		}
		else {
			$dailySchedule = 'During hour ' . $runTime;
		}

		$runTime = ConfigHandler::Get('/cron/run/weekly');
		$options = [
			'-1' => 'Any day of the week',
			'1' => 'Monday',
			'2' => 'Tuesday',
			'3' => 'Wednesday',
			'4' => 'Thursday',
			'5' => 'Friday',
			'6' => 'Saturday',
			'7' => 'Sunday',
		];
		$weeklySchedule = $options[ $runTime ] ?? 'Invalid Schedule';

		$runTime = ConfigHandler::Get('/cron/run/monthly');
		if($runTime === -1) {
			$monthlySchedule = 'Any day';
		}
		else {
			$monthlySchedule = 'On day ' . $runTime;
		}



		$view->title = 't:STRING_LATEST_CRON_RESULTS';
		$view->assign('crons', $crons);
		$view->assign('enabled', $cronsEnabled);
		$view->assign('last_run', $latestCron);
		$view->assign('last_run_delta', DateTime::NowGMT() - $latestCron);
		$view->assign('daily_schedule', $dailySchedule);
		$view->assign('weekly_schedule', $weeklySchedule);
		$view->assign('monthly_schedule', $monthlySchedule);
	}
}
