<?php
/**
 * Class file for the controller FaviconController
 *
 * @package Favicon
 * @author Charlie Powell <charlie@evalagency.com>
 */

use Core\ConfigHandler;
use Core\Controller;
use Core\Filestore\Factory;
use Core\Forms\Form;
use Core\Forms\TabsGroup;
use Core\View;


class FaviconController extends Controller {

	/**
	 * @core-access-string g:admin
	 *
	 * @throws Exception
	 */
	public function admin(){
		$view = $this->getView();
		$request = $this->getRequest();

		$image = ConfigHandler::Get('/favicon/image');

		$form = new Form();
		$form->set('callsmethod', 'AdminController::_ConfigSubmit');
		
		$basic = new TabsGroup(['name' => 'basic', 'title' => 'Basic Settings']);
		$advanced = new TabsGroup(['name' => 'advanced', 'title' => 'Advanced Settings']);

		$basic->addElement(ConfigHandler::GetConfig('/favicon/image')->getAsFormElement());

		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-512')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-256')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-196')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-192')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-180')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-120')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-96')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-76')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-64')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-60')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-48')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-32')->getAsFormElement());
		$advanced->addElement(ConfigHandler::GetConfig('/favicon/image-16')->getAsFormElement());
		
		$form->addElement($basic);
		$form->addElement($advanced);
		
		$form->addElement('submit', ['value' => t('STRING_SAVE')]);

		$view->title = 'Site Favicon';
		$view->assign('current', $image);
		$view->assign('form', $form);
	}

	/**
	 * Simple method to handle any legacy call to favicon.ico
	 */
	public function index() {
		$view = $this->getView();

		$image = ConfigHandler::Get('/favicon/image');
		$file = Factory::File($image);

		$view->contenttype = 'image/png';
		$view->record = false;
		$view->mode = View::MODE_NOOUTPUT;

		// Fix Bug #562, Favicon "none" option.
		// Do not render anything if no icon is selected.
		if(!$image){
			return;
		}
		if(!$file->exists()){
			return;
		}

		$file->getPreviewFile('32x32!')->sendToUserAgent();
	}

	/**
	 * Hook to add in the necessary favicon ViewMeta attribute into the View.
	 *
	 * The ViewMeta_favicon handles the rest of the magic.
	 *
	 * @param View $view
	 */
	public static function PageHook(View $view) {
		$view->addMetaName('favicon', null);
	}

	public static function GetConfigurationLinks() {
		$ret = [];
		$user = \Core\user();
		$canManage = $user->checkAccess('g:admin');

		if($canManage) {
			$ret[] = [
				'title' => 'Site Favicon',
				'link' => '/favicon/admin',
				'description' => 'Site favicon is a small icon that is used in browser tabs, mobile icons, desktop icons, and the sidebar menu header.'
			];
		}

		return $ret;
	}
}
