<?php

/**
 * Geo Province form input for picking country+province.
 * 
 * Pulls dynamically from the database for real results.
 */

use Core\Forms\FormElement;
use Core\Templates\Template;


class GeoProvinceFormInput extends FormElement {
	public function  __construct($atts = null) {
		parent::__construct($atts);

		// Some defaults
		$this->classnames[] = 'formelement';
		$this->classnames[] = 'geoprovinceforminput';
		//$this->_validattributes     = array('accesskey', 'dir', 'disabled', 'id', 'lang', 'name', 'required', 'tabindex', 'rows', 'cols');
	}

	public function render(){
		// Required to be set to inform the calling controller that this element *was* infact rendered,
		// and an empty value is literally just that.
		// Otherwise the calling system may think it was never rendered and use the default value.
		$this->rendered = true;

		
		$v = $this->get('value');
		
		if(!$v){
			// Default?...
			$v = REMOTE_COUNTRY . ':' . REMOTE_PROVINCE;
		}
		
		// Split them into country and province.
		list($country, $province) = explode(':', $v);

		// Get the provinces for the given selected country, (to save an ajax call)
		$provinces = GeoLookup\Lookup::GetProvinces($country);
		$countries = GeoLookup\Lookup::GetCountries();

		// Convert the provinces to JSON data so the javascript
		// can use it as if it had loaded from the server.
		$provincejs = [];
		foreach($provinces as $p){
			/** @var GeoLookup\Result $p */
			$provincejs[] = [
				'country' => $p->country,
				'code'    => $p->province,
				'name'    => $p->getProvinceName()
			];
		}

		$file = $this->getTemplateName();

		$tpl = Template::Factory($file);

		$tpl->assign('province', $province);
		$tpl->assign('country', $country);
		$tpl->assign('provinces', $provinces);
		$tpl->assign('province_json', json_encode($provincejs));
		$tpl->assign('countries', $countries);
		$tpl->assign('element', $this);
		$tpl->assign('req', $this->get('required'));

		return $tpl->fetch();
	}
}
