var NavigationManager = null;

(function () {
	var addcounter = 0,
		options = {

		},
		dialogs = {
			'int': null,
			'ext': null,
			'none':null
		},
		extractpagename,
		$renderarea,
		Dialog,
		sorttimeout;

	extractpagename = function ($select) {
		// Find the active option and retrieve the pagename from its label.
		// Required because pagenames may be in the format of
		// Home Page &raquo; Core+ &raquo; Why Choose Core+ Framework? ( /why-choose-core-plus )

		var $opt = $select.find('option:selected'),
			s = Core.Strings.trim($opt.html());

		return s.replace(/ \(.*\)$/, '').replace(/.*» (.*)$/, '$1')
	};

	Dialog = function (type, typetitle) {
		var self = this;

		//// PUBLIC PROPERTIES
		this.$dialog = null;

		//// PUBLIC METHODS

		/**
		 * Extend this to do any sanity checks before saving.
		 * Return false to invalidate the save.
		 */
		this.beforesave = function () {
			return true;
		};

		this.open = function ($el) {
			var id, title, url, target, windowtitle;

			if (typeof $el == 'undefined') {
				// It's a new window, not too many options here.
				windowtitle = 'New ' + typetitle;
			}
			else {
				id = $el.data('id');

				title = $el.find('.entry-title').val();
				url = $el.find('.entry-url').val();
				target = $el.find('.entry-target').val();

				windowtitle = 'Edit ' + title;
			}

			// Set the data
			this.$dialog.find('input[name=id]').val(id);
			this.$dialog.find('input[name=title]').val(title);
			this.$dialog.find(':input[name=url]').val(url);
			this.$dialog.find(':input[name=target]').val(target);

			// Open the dialog
			this.$dialog.show().dialog({
				modal:   true,
				autoOpen:false,
				title:   windowtitle,
				width:   '500px'
			}).dialog('open');
		};

		this.save = function () {
			var dat = {
				id:    this.$dialog.find(':input[name=id]').val(),
				type:  type,
				url:   this.$dialog.find(':input[name=url]').val(),
				target:this.$dialog.find(':input[name=target]').val(),
				title: this.$dialog.find('input[name=title]').val()
			};

			// Sanity checks.
			if (!dat.title) {
				alert('You must include a title.');
				return false;
			}

			// User-definable sanity check.
			if (!this.beforesave(dat)){
				return false;
			}

			// New link
			if (!dat.id) {
				NavigationManager.addEntry(dat);
			}
			// Existing link
			else {
				NavigationManager.editEntry(dat);
			}

			this.$dialog.dialog('close');
		};

		//// CONSTRUCTOR LOGIC

		this.$dialog = $('.add-entry-options-' + type);

		// This is the trigger for opening this dialog.
		$('.add-entry-' + type + '-btn').click(function () {
			self.open();

			return false;
		});

		// Save button of course
		this.$dialog.find('.submit-btn').click(function () {
			self.save();

			return false;
		});
	};

	NavigationManager = {
		init:     function () {

			// Setup some private variables real quick.
			$renderarea = $('#entry-listings');
			dialogs['int'] = new Dialog('int', 'Internal Link');
			dialogs['ext'] = new Dialog('ext', 'External Link');
			dialogs['none'] = new Dialog('none', 'Text Label');

			// By default, set the title of the link to the title of the page, for convenience.
			dialogs['int'].$dialog.find('select[name=url]').change(function () {
				dialogs['int'].$dialog.find('input[name=title]').val(extractpagename($(this)));
			});

			// External links have additional sanity checks.
			dialogs['ext'].beforesave = function (dat) {
				// external links should be resolved fully!
				if (dat.url.indexOf('://') === -1) {
					dat.url = 'http://' + dat.url;
				}

				return true;
			};

			// Do the actual sortable logic
			$renderarea.nestedSortable({
				disableNesting:      'no-nest',
				forcePlaceholderSize:true,
				handle:              'a.drag-entry-link',
				helper:              'clone',
				items:               'li.entry',
				opacity:             .6,
				placeholder:         'placeholder',
				tabSize:             25,
				tolerance:           'pointer',
				toleranceElement:    '> div'
			});

			// Edit links, should edit!
			$renderarea.delegate('a.edit-entry-link', 'click', function () {
				var $li = $(this).closest('li'),
					type = $li.find('.entry-type').val();

				dialogs[type].open($li);

				return false;
			});

			// And delete links should delete.
			$renderarea.delegate('a.delete-entry-link', 'click', function () {
				var $li = $(this).closest('li'),
					id = $li.data('id');

				// Remove this item and any children.
				$li.find('.entry-status').val('deleted');
				$li.hide();

				return false;
			});

			// Capture the form submission, I need to scan through and update the parentids as necessary.
			$renderarea.closest('form').submit(function () {
				$renderarea.find('li.entry').each(function () {
					var $this = $(this),
						$parent = $this.parent().closest('li.entry'),
						parentid = ($parent.length > 0) ? $parent.data('id') : 0;

					$this.find('.entry-parent').val(parentid);
				});
			});
		},

		// Add an entry to the page, with all the information setup appropriately.
		addEntry: function (data) {
			var
				parent = data.parent || 0,
				payload,
				$target;

			if(typeof(data) === 'undefined') {
				data = {
					id: 'new-' + (++addcounter),
					status: 'new'
				};
			}
			else if(data.id === '') {
				data.id = 'new-' + (++addcounter);
				data.status = 'new';
			}

			// Create the payload first, this is just basic HTML.
			payload = Core.Strings.template($('#navigation-entry-template').html(), data);


			// Should this payload be attached onto the root node, or the parent node?
			if (parent) {
				// Look up that parent first.
				$target = $renderarea.find('li[data-id=' + parent + ']');
				// No target? Reset to the parent.
				if (!$target.length) {
					$target = $renderarea;
				}
				else {
					// Find the child OL for this target, create one if it doesn't exist.
					if (!$target.children('ol').length) $target.append('<ol/>');
					$target = $target.children('ol');
				}
			}
			else {
				// Easy enough here.
				$target = $renderarea;
			}

			$target.append(payload);
		},

		// Add an entry to the page, with all the information setup appropriately.
		editEntry:function (data) {
			var
				id = data.id,
				$target = $renderarea.find('#entry-' + id);

			$target.find('> .entry-url').val(data.url);
			$target.find('> .entry-target').val(data.target);
			$target.find('> .entry-title').val(data.title);

			$target.removeClass('target-').removeClass('target-_BLANK').addClass('target-' + data.target);
		}
	};
})();

$(NavigationManager.init);