<?php
/**
 * File for class SimpleSitePassword definition in the Ticketing Application project
 * 
 * @package SecuritySuite
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20140214.2130
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace SecuritySuite;

use Core\ConfigHandler;


/**
 * Helper for the security configuration option /security/site_password.
 *
 * Provides a simple hook to check to see if the config is populated and HTTP authentication passes.
 *
 * This is essentially a built-in .htpasswd system.
 *
 * @package SecuritySuite
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class SimpleSitePassword {
	public static function Check(){

		$pw = ConfigHandler::Get('/security/site_password');

		if($pw == ''){
			return true;
		}

		if(EXEC_MODE != 'WEB'){
			// Simple site passwords only work when on the web.
			return true;
		}

		if(isset($_SERVER['PHP_AUTH_PW'])){
			// Standard auth in MOD mode, works without requiring workarounds
			$userpw = $_SERVER['PHP_AUTH_PW'];
		}
		elseif(isset($_SERVER['REDIRECT_HTTP_AUTHORIZATION'])) {
			// CGI mode does not allow for the direct access of PHP_AUTH.
			$auth = filter_var($_SERVER['REDIRECT_HTTP_AUTHORIZATION'], FILTER_SANITIZE_STRING);
			if(stripos($auth, 'basic') === 0) {
				// Basic auth request!
				list($user, $userpw) = explode(':', base64_decode(substr($auth, 6)));
			}
			else {
				$userpw = null;
			}
		}
		else{
			// No auth provided
			$userpw = null;
		}

		if($userpw != $pw){
			header('WWW-Authenticate: Basic realm="' . SITENAME . '"');
			header('HTTP/1.0 401 Unauthorized');
			echo 'Access to ' . SITENAME . ' requires a password.';
			exit;
		}

		// Else everything.
		return true;
	}
}
