
setInterval(function() {
	$('.listing-table-2018').each(function(){
		if(
			$(this).find('.auto-refresh-link').hasClass('active')
		) {
			this.reloadData(null, true);
		}
	});
}, 15000);


// Available as a stand-alone call for dynamically loaded tables
$(window).change(function() {
	// Initial setup of the tables, namely directional arrows for sorting and default display options.
	$('.listing-table-2018').each(function(){
		var table          = this,
			$table         = $(table),
			sidebarLoaded  = $table.data('sidebar-events-loaded') || false;


		// Attach all necessary functions onto the table


		/**
		 * Load new parameters onto this table and reload the updated data.
		 * 
		 * @param string params
		 */
		table.loadParams = function(params) {
			console.log(params);
			if(params.indexOf(Core.ROOT_URL) === 0) {
				// Are parameters a fully resolved URL?
				params = '/' + params.substr(Core.ROOT_URL.length);
			}
			
			console.log(params);
			if(params.indexOf(Core.Url) === 0) {
				// The params start with the base page URL, this is also unnecessary.
				params = params.substr(Core.Url.length);
			}
			
			console.log(params);
			if(params[0] !== '?') {
				// Ensure params starts with a '?'
				params = '?' + params;
			}
			
			if(Core.Url.indexOf('?') === -1){
				Core.Url += params;
			}
			else{
				Core.Url = Core.Url.substr(0, Core.Url.indexOf('?')) + params;
			}
			
			table.reloadData();
		}

		/**
		 * Reload and redraw this table data from the server
		 * 
		 * @param string params     List of GET parameters to use for the request
		 * @param bool   background Set to TRUE to issue a background request, less intrusive for the UX
		 */
		table.reloadData = function(params, background) {
			var $container  = $table.find('.listing-table-data-container'),
				$data       = $table.find('.listing-table-data'),
				$link       = $table.find('.auto-refresh-link'),
				$icon       = $link.find('i.icon'),
				loadingText = $table.find('.listing-table-loading-template').html(),
				autoReload  = $link.hasClass('active'),
				path        = Core.ROOT_URL + $table.data('baseurl').substr(1);


			if(typeof(background) === 'undefined' || !background) {
				// If background is omitted or set to true, (default bahaviour), render a loading... dialog to provide hints to the user.
				$data.html(loadingText)
			}
			else {
				// Low interference UX change
				$icon.addClass('icon-spin');
			}
			
			
			if(path.indexOf('?') !== -1) {
				// Trim this off the path; it may be provided if the incoming URL had a query string.
				path = path.substr(0, path.indexOf('?'));
			}
			
			// If there were parameters requested, translate them as necessary.
			if(typeof(params) === 'undefined' || params === '' || params === null) {
				params = '';
			}
			else if(params.indexOf('://') !== -1 && params.indexOf('?') !== -1) {
				// Trim off any fully resolved URL, we just want the parameters
				params = params.substr(params.indexOf('?'));
			}
			else if(params.indexOf('://') !== -1) {
				console.error('Refusing to load URL that does not match the current page!', params);
				params = '';
			}
			else {
				params = '?' + params;
			}


			// Container is loaded into to prevent duplicate nested IDs.
			$container.load(
				path + params + ' #' + $data.attr('id'),
				function(){
					if(autoReload) {
						$table.find('.auto-refresh-link').addClass('active');
					}

					table.updateSortIcons();
					table.updateTooltips();
				}
			);
		};

		
		table.updateTooltips = function() {
			$table.find('.listing-table-pagination-and-actions a.button').tooltip();
		};

		/**
		 * Add the class to indicate the currently selected sort icon
		 */
		table.updateSortIcons = function() {
			$table.find('[data-sortkey="' + $table.data('table-sortkey') + '"]')
				.find('.icon-sort-' + $table.data('table-sortdir'))
				.addClass('icon-sort-current');
		};

		/**
		 * Bind all the sidebar events, these generally do not get recreated with data loads.
		 */
		table.sidebarEvents = function() {
			// Filter events

			$table.on('submit', '.listing-filters form', function(e) {
				e.preventDefault();  //prevent form from submitting
				table.reloadData($table.find('.listing-filters form').serialize());
				return false;
			});

			$table.on('click', '.apply-filters', function(e) {
				e.preventDefault();  //prevent form from submitting
				table.reloadData($table.find('.listing-filters form').serialize());
				return false;
			});


			// Display Option events

			$table.on('change', '.pagination-limit-selector', function() {
				table.reloadData('limit=' + this.options[this.selectedIndex].value);
				return false;
			});

			$table.on('submit', '.listing-display-options form', function(e) {
				e.preventDefault();  //prevent form from submitting
				table.reloadData($table.find('.listing-display-options form').serialize());
				return false;
			});

			$table.on('click', '.apply-columns', function(e) {
				e.preventDefault();  //prevent form from submitting
				table.reloadData($table.find('.listing-display-options form').serialize());
				return false;
			});

			$table.on('submit', '.bulk-actions form', function(e) {
				var $form = $(this), $checked = $table.find('.bulk-select:checked');

				if(!$checked.length) {
					alert('Please select items first.');
					e.preventDefault();  //prevent form from submitting
					return false;
				}

				$checked.each(function() {
					$form.append($(this));
				});

				//return false;
			});


			// Report events
			$table.on('click', '.report-view', function(){
				var $this = $(this),
					$dialog = $('<div class="listing-table-report-viewer"></div>').appendTo('body'),
					$form = $this.closest('form'),
					url = window.location.href,
					title = $this.data('title') || 'Report';

				if(url.indexOf('?') !== -1) {
					// Trim anything in the query string, we just want the URL.
					url = url.substr(0, url.indexOf('?'));
				}

				// Add the form elements
				url = url + '?' + $form.serialize() + '&format=html';

				$dialog.html('Loading Report... <i class="icon icon-spin icon-spinner"></i>');
				$dialog.load(url);
				$dialog.dialog({
					resizable: true,
					modal: true,
					position: { my: "center", at: "top center", of: window },
					width: '75%',
					title: title,
					close: function(){
						$dialog.dialog('destroy');
					}
				});

				// Prevent page jumping.
				return false;
			});

			$table.on('click', '.report-download', function(){
				var $this = $(this),
					$form = $this.closest('form'),
					url = window.location.href;

				if(url.indexOf('?') !== -1) {
					// Trim anything in the query string, we just want the URL.
					url = url.substr(0, url.indexOf('?'));
				}

				// Add the form elements
				url = url + '?' + $form.serialize() + '&format=pdf';

				window.location.href = url;

				// Prevent page jumping.
				return false;
			});
		};


		// The columns may have changed, redraw the results initially!
		this.updateSortIcons();
		this.updateTooltips();

		if(!sidebarLoaded) {
			this.sidebarEvents();
		}


		// Table header and action events

		$table.on('click', '.print-link', function() {
			window.print();
			return false;
		});

		$table.on('click', '.auto-refresh-link', function() {
			var $this = $(this);
			
			if($this.hasClass('active')) {
				$.notify('Deactivated auto-refresh', 'info');
				$(this).toggleClass('active');
			}
			else {
				$.notify('Activated auto-refresh', 'info');
				$(this).toggleClass('active');
			}
			
			return false;
		});
		
		$table.on('click', '.sort-icon', function() {
			var $icon = $(this), 
				newkey = $icon.data('key'), 
				newdir = $icon.data('dir'),
				req = 'sortkey=' + newkey + '&sortdir=' + newdir;
			
			$table.data('table-sortkey', newkey);
			$table.data('table-sortdir', newdir);
			
			table.reloadData(req);
			return false;
		});



		
		$table.on('click', '.pagination-pager a', function() {
			table.reloadData(this.href);
			return false;
		});


		// Bulk select events

		$table.on('click', '.bulk-select', function(e) {
			var $actions = $table.find('.bulk-actions');

			if($actions.hasClass('collapsed')) {
				// If the actions is not already expanded, expand it to provide the user hinting about what checking items controls.
				$actions.find('.fieldset-title').click();
			}
		});

		$table.on('click', '.bulk-select-all', function(e) {
			var $actions = $table.find('.bulk-actions'),
				isChecked = $(this).is(':checked');

			$table.find('.bulk-select').prop('checked', isChecked);

			if(
				($actions.hasClass('collapsed') && isChecked) ||
				(!$actions.hasClass('collapsed') && !isChecked)
			) {
				// If the actions is not already expanded, expand it to provide the user hinting about what checking items controls.
				$actions.find('.fieldset-title').click();
			}
		});




		// LASTLY
		$table.data('sidebar-events-loaded', true);
	});
});

