/**
 * Retrieve updates from any repositories and display new packages to the user for installation.
 *
 * @type {Object}
 */
(() => {
	let packageTemplate = document.getElementById('package-template').innerHTML,
		packageTarget = document.getElementById('package-render-area'),
		loadingText = document.getElementById('package-loading-text'),
		noPackagesText = document.getElementById('package-none-available'),
		packageSearch = document.getElementById('package-search');

	fetch(Core.ROOT_URL + 'updater/getpackages.json')
		.then(res => {
			return res.json();
		})
		.then(data => {
			let html = '';

			loadingText.style.display = 'none';

			if(!data.packages.length) {
				noPackagesText.style.display = 'block';
			}

			for(let i = 0; i < data.packages.length; i++) {
				html += packageTemplate.template(data.packages[i]);
			}

			packageTarget.innerHTML = html;
		});

	packageSearch.addEventListener('keyup', evt => {
		let search = evt.target.value.toLowerCase();

		document.querySelectorAll('#package-render-area .collection-item').forEach(el => {
			let match = false;


			if(!search) {
				// Default for an empty search value
				match = true;
			}
			else {
				// Something's set, actually look for it.
				// Search in a few locations for this text.
				if(el.querySelector('.collection-item-title').innerHTML.toLowerCase().match(search) !== null) {
					match = true;
				}

				if(el.querySelector('.collection-item-description').innerHTML.toLowerCase().match(search) !== null) {
					match = true;
				}
			}

			if(match) {
				el.style.display = 'flex';
			}
			else {
				el.style.display = 'none';
			}
		});
	});
})();