/**
 * The main updater object, will contain public functions useful for external scripts to tie into
 * (well, scripts that are on the updater/index page...)
 *
 * @type {Object}
 */
(() => {
	fetch(Core.ROOT_URL + 'updater/getupdates.json')
		.then(res => {
			return res.json();
		})
		.then(data => {
			let packages = {};

			// Group the returned data by their keys for quick lookups
			for(let i = 0; i < data.updates.length; i++) {
				packages[ data.updates[i].type + '-' + data.updates[i].key ] = data.updates[i];
			}

			// Sift over every update link and update its state to match the resolved data.
			document.querySelectorAll('.update-link').forEach(el => {
				let key = el.dataset.type + '-' + el.dataset.key,
					icon = el.querySelector('i'),
					span = el.querySelector('span'),
					url = Core.ROOT_URL + 'updater/' + el.dataset.type + '/install/' + el.dataset.key;

				// Stop the spinner; data has been returned.
				icon.classList.remove('icon-spin', 'icon-spinner');

				if(typeof(packages[key]) === 'undefined') {
					span.innerHTML = 'No Repo Available';
					el.classList.add('disabled');
				}
				else if(packages[key].update) {
					icon.classList.add('icon-download');
					span.innerHTML = 'Update to ' + packages[key].version;
					el.href = url + '?version=' + packages[key].version;
				}
				else {
					icon.classList.add('icon-redo');
					span.innerHTML = 'Reinstall ' + packages[key].version;
					el.href = url + '?version=' + packages[key].version;
				}
			});
		})
})();
