<?php
/**
 * DESCRIPTION
 *
 * @package Core
 * @since 2.5.4
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\Controller;
use Core\Filestore\Factory;
use Core\View;


class FileController extends Controller {

	public function preview() {
		$view = $this->getView();
		$request = $this->getRequest();

		// This is designed to only return data!
		$view->mode = View::MODE_NOOUTPUT;
		// And it shouldn't be recorded in navigation.
		$view->record = false;

		// The filename should be something like files/public/blah... or files/assets/foo...
		// This will get fed into the core system and checked internally.
		$filename = $request->getParameter('f');

		// Was there a resize-to dimension requested?
		$d = $request->getParameter('d');


		// The inbound filename must be in the format of base64:[b64data].
		if (strpos($filename, 'base64:') !== 0) {
			\Core\log_notice('Invalid request made for /file/preview!  Expecting: [base64:*b64data*], Received: [' . $filename . ']');

			$file = Factory::File('assets/images/placeholders/noimage.svg');
			$file->sendToUserAgent();
			return;
		}

		// This preview ONLY supports assets and public files!
		// This is a security precaution.
		$filename = base64_decode(substr($filename, 7));

		if(
			// Fully resolved filenames are not allowed
			$filename[0] === '/' ||
			// Files containing '../' are not allowed
			strpos($filename, '../') !== false ||
			// The path must be in one of public or assets.
			(strpos($filename, 'public/') !== 0 && strpos($filename, 'assets/') !== 0 && strpos($filename, 'asset/') !== 0)
		){
			\Core\log_notice('Invalid file requested: ' . $filename);

			$file = Factory::File('assets/images/placeholders/noimage.svg');
			$file->sendToUserAgent();
			return;
		}
		
		$file = Factory::File($filename);
		if(!$file->exists()){
			\Core\log_notice('File not found for /file/preview!  Looking For: [' . $file->getFilename('') . ' (' . $filename . ') ]');

			$file = Factory::File('assets/images/placeholders/noimage.svg');
			$file->sendToUserAgent();
			return;
		}

		// And this will render it to the browser.
		$preview = $file->getPreviewFile($d);
		$preview->sendToUserAgent();
	}
}

