<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core;
use DOMElement;
use PageModel;

/**
 * Description of ComponentPageDefinition
 *
 * @author charlie
 */
class ComponentPageDefinition {
	
	public $access;
	public $admin = false;
	public $baseurl;
	public $component;
	public $image;
	public $indexable;
	public $editurl;
	public $group;
	public $parenturl;
	public $rewriteurl;
	public $selectable = false;
	public $title;
	public $fuzzy;
	private $_model;
	
	public function __construct(DOMElement $node, string $component) {
		$this->component  = $component;
		$this->baseurl    = $node->getAttribute('baseurl');
		$this->rewriteurl = $node->getAttribute('rewriteurl') ?: $this->baseurl;
		$this->admin      = ($node->getAttribute('admin') === '1');
		$this->title      = $node->getAttribute('title');
		$this->access     = $node->getAttribute('access');
		$this->editurl    = $node->getAttribute('editurl');
		$this->image      = $node->getAttribute('image');
		$this->parenturl  = $node->getAttribute('parenturl');
		$this->fuzzy      = ($node->getAttribute('fuzzy') === '1');
		
		if($this->admin){
			// Only admin pages support groups.
			$this->group = $node->getAttribute('group');
		}
		
		if($node->getAttribute('selectable') === '1'){
			$this->selectable = true;
		}
		elseif($node->getAttribute('selectable') === ''){
			// Selectable is blank, means it's left up to if it's an admin page or not.
			$this->selectable = !$this->admin;
		}
		else{
			$this->selectable = false;
		}
		
		if($node->getAttribute('indexable') === '1'){
			$this->indexable = true;
		}
		elseif($node->getAttribute('indexable') === '0'){
			$this->indexable = false;
		}
		else{
			$this->indexable = $this->selectable;
		}
	}

	public function getModel() : PageModel {
		if($this->_model === null) {
			$this->_model = new PageModel(-1, $this->baseurl);
		}

		return $this->_model;
	}

	/**
	 * Install this defined page into the system and return true if changes were detected.
	 *
	 * @return bool
	 * @throws \Exception
	 */
	public function install() : bool {
		// These are always global pages.
		$model = $this->getModel();
		
		if($model->checkIsNew() || $this->admin){
			// Some keys only get set on new objects so as to not overwrite user settings.
			$model->setFromArray([
				'rewriteurl' => $this->rewriteurl,
				'title'      => $this->title,
				'parenturl'  => $this->parenturl,
			]);
		}
		
		// Standard overwrites that apply to both existing and new pages.
		$model->setFromArray([
			'access'      => $this->access,
			'admin'       => $this->admin,
			'admin_group' => $this->group,
			'selectable'  => $this->selectable,
			'indexable'   => $this->indexable,
			'component'   => $this->component,
			'editurl'     => $this->editurl,
			'fuzzy'       => $this->fuzzy,
		]);
		
		if($this->image){
			$model->setMeta('image', $this->image);
		}
		
		return ($model->save());
	}
	
	/**
	 * Uninstall this defined page from the system
	 * 
	 * @return bool
	 */
	public function uninstall() : bool {
		// These are always global pages.
		$model = new PageModel(-1, $this->baseurl);
		return ($model->delete());
	}
}
