<?php
/**
 * File for class Schema definition in the coreplus project
 * 
 * @package Core\Datamodel
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20131022.1655
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Datamodel\Columns;
use Core\Model;

class SchemaColumn_float extends SchemaColumn {
	public function __construct(){
		// Defaults
		$this->typecast = 'float';
		$this->type = Model::ATT_TYPE_FLOAT;
		$this->default = 0.0;
		$this->formAttributes['type'] = 'number';
		$this->formAttributes['access'] = '*';
		$this->formAttributes['autocomplete'] = 'off';
	}

	/**
	 * Get an array of the form element attributes for this column.
	 *
	 * @return array
	 */
	public function getFormElementAttributes() {
		// Provide some defaults if not otherwise set.

		if($this->precision !== null) {
			list($a, $b) = array_map('trim', explode(',', $this->precision));

			if(!isset($this->formAttributes['step'])) {
				// Ensure the stepping if not requested already, calculated from the precision option
				$this->formAttributes['step'] = pow(10, 0-$b);
			}

			if(!isset($this->formAttributes['max'])) {
				// Ensure the max value is set, calculated from the precision option
				$this->formAttributes['max'] = pow(10, $a) - 1;
			}

			if(!isset($this->formAttributes['min'])) {
				// Ensure the min value is set, just 0 by default.
				$this->formAttributes['min'] = 0;
			}
		}

		// Set the pattern for fallback support for browsers that don't support type="number"
		if(!isset($this->formAttributes['pattern'])) {
			$this->formAttributes['pattern'] = '/^[+-]?\d+(\.\d+)?$/';
		}

		return parent::getFormElementAttributes();
	}
}