<?php
/**
 * Description of ContentSVG
 *
 * Provides useful extra functions that can be done with an XML file.
 *
 * @package Core\Filestore\Contents
 * @since 2.2.0
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Filestore\Contents;

use Core\Filestore\ContentsInterface;
use Core\Filestore\File;
use Core\Parser\SVGMeta\SVGMetaParser;
use Core\XMLLoader;

class ContentSVG implements ContentsInterface {
	private $_file = null;

	public function __construct(File $file) {
		$this->_file = $file;
	}

	public function getContents() {
		return $this->_file->getContents();
	}

	/**
	 * Get the associated XMLLoader object for this data
	 *
	 * @return XMLLoader
	 */
	public function getLoader(){
		$xml = new XMLLoader();
		$xml->setRootName('svg');
		$xml->loadFromFile($this->_file);
		return $xml;
	}

	/**
	 * Get the dimensions of this SVG as a string of WidthxHeight
	 *
	 * @return string
	 */
	public function getDimensions() : string {
		$loader = $this->getLoader();
		$root = $loader->getRootDOM();
		$viewbox = $root->getAttribute('viewBox');

		if($viewbox) {
			$parts = explode(' ', $viewbox);
			$w = $parts[2] - $parts[0];
			$h = $parts[3] - $parts[1];
		}
		else {
			$w = $root->getAttribute('width');
			$h = $root->getAttribute('height');
		}

		return (int)$w . 'x' . (int)$h;
	}

	/**
	 * Get the exif data of this image.
	 *
	 * @return SVGMetaParser
	 */
	public function getExif() : SVGMetaParser {
		$parser = new SVGMetaParser();
		$parser->parseFile($this->_file);
		return $parser;
	}
}

