<?php
/**
 * Class file for FormTextInput
 *
 * @package Core\Forms
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Forms;


/**
 * Class FormNumberInput provides a simple text inputs (specific to numbers)
 *
 * @package Core\Forms
 */
class NumberInput extends FormElement {
	public function  __construct($atts = null) {
		parent::__construct($atts);

		// Some defaults
		$this->_attributes['class'] = 'formelement formnumberinput';
		$this->_validattributes     = [
			'accesskey',
			'autocomplete',
			'disabled',
			'height',
			'id',
			'name',
			'placeholder',
			'readonly',
			'required',
			'size',
			'style',
			'tabindex',
			'value',
			'width',
			'list',
			'max',
			'min',
			'step',
			'pattern', // Pattern is supported for fallback for browsers that do not support type="number"
		];
	}

	public function getInputAttributes(){
		if($this->get('autocomplete') && !($this->get('autocomplete') == 'off' || $this->get('autocomplete') == 'on')){
			// These get handled by the smarty snippet.
			if(($k = array_search('name', $this->_validattributes))){
				unset($this->_validattributes[$k]);
			}
			if(($k = array_search('value', $this->_validattributes))){
				unset($this->_validattributes[$k]);
			}
		}

		return parent::getInputAttributes();
	}
}