<?php


namespace Core\Forms;


class PageInsertableGroup extends FormElement {
	public function render() {
		// Required to be set to inform the calling controller that this element *was* infact rendered,
		// and an empty value is literally just that.
		// Otherwise the calling system may think it was never rendered and use the default value.
		$this->rendered = true;

		return '<script>' .
			'window.PageEditorGroups = window.PageEditorGroups || { };' .
			'window.PageEditorGroups["' . $this->get('data-group') . '"] = ' . json_encode($this->getValue()) . ';' .
			'</script>';
	}

	public function setValue($value) {
		$group = $this->get('group_parent');
		$elements = [];
		$newValue = [];

		if(!is_array($value)) {
			$value = [];
		}

		if($group && $group instanceof FormGroup) {
			// Grab all the TEMPLATE elements contained herein.
			// These will provide the validation logic and any default conversions
			// that may come along with the given form element type.
			$groupElements = $group->getElements(true);
			foreach($groupElements as $elementTemplate) {
				$elementName = $elementTemplate->get('name');
				// Name should consist of something similar to
				// page[insertables][callouts5299][__INSERTABLEGROUPTEMPLATE][testblerp]
				$elementName = preg_replace('/.*\[([a-zA-Z0-9_]*)\]$/', '$1', $elementName);

				$elements[ $elementName ] = $elementTemplate;
			}

			foreach($value as $groupIDX => $groupData) {
				$groupDataProcessed = [];

				foreach($groupData as $elementKey => $elementValue) {
					if(!isset($elements[ $elementKey ])) {
						continue;
					}

					/** @var FormElement $testElement */
					$testElement = clone($elements[ $elementKey ]);
					$testElement->set('name', str_replace('__INSERTABLEGROUPTEMPLATE', $groupIDX, $testElement->get('name')));
					$testElement->setValue($elementValue);

					if($testElement->hasError()) {
						$this->_error = $testElement->getError();
						return false;
					}

					$groupDataProcessed[ $elementKey ] = $testElement->getValue();
				}

				$newValue[] = $groupDataProcessed;
			}

			$this->_attributes['value'] = $newValue;
			return true;
		}
		else {
			// Nothing linked.... I guess it may be from the initialization.
			$this->_attributes['value'] = $value;
			return true;
		}
	}
}