<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Installer\Checks;
use Core\Installer\InstallHelper;


/**
 * Checks if all contents of the given directory, (sans .htaccess), are writeable
 * by the current user.
 * 
 * @author charlie
 */
class DirectoryContentsWriteableCheck implements CheckInterface {
	private $_path;
	
	public function __construct($path){
		$this->_path = $path;
	}
	
	/**
	 * Get the title of this check
	 * @return string
	 */
	public function getTitle() : string {
		return 'Directory ' . basename($this->_path) . ' and its contents are writeable';
	}
	
	/**
	 * Run the test for this check
	 * @return bool True if passed, False otherwise
	 */
	public function test() : bool {
		
		// use the system's find command to locate any non-writeable file contained herein.
		$count = exec('find ' . escapeshellarg($this->_path) . ' -not -writable -not -name .htaccess | wc -l');
		
		return $count <= 0;
	}

	public function testExtended(): string {
		return $this->test() ? self::STATUS_PASSED : self::STATUS_FAILED;
	}
	
	/**
	 * Get additional details that may help the user fix the problem
	 * @return string
	 */
	public function getDetails() : string {
		$user = InstallHelper::GetUser();
		
		return 'Ensure that the user "' . $user . '" can write to the directory ' . $this->_path;
	}

	/**
	 * Get additional details about this check, but not necessarily about the fix.
	 * @return string
	 */
	public function getInformation() : string {
		return '';
	}
	
	/**
	 * Get the fix for this problem that instructs the user how to fix it
	 * @return string
	 */
	public function getFix(): string {
		$user = InstallHelper::GetUser();
		
		if (defined('DEVELOPMENT_MODE') && DEVELOPMENT_MODE) {
			// DEV mode is in use here; provide less restrictive permissions.
			return 'chown -R ' . $user . ' ' . escapeshellarg($this->_path) . NL .
				'chmod -R ug+rwX ' . escapeshellarg($this->_path);
		}
		else{
			// LIVE mode, return fully restrictive permissions.
			return 'chown -R ' . $user . ':' . $user . ' ' . escapeshellarg($this->_path) . NL .
				'chmod -R u+rwX,go-rwx ' . escapeshellarg($this->_path);
		}
	}
}
