<?php


namespace Core\Installer\Checks;


class PHPVersionCheck implements CheckInterface {

	public function getTitle(): string {
		return 'PHP Version';
	}

	public function test(): bool {
		return $this->testExtended() !== self::STATUS_FAILED;
	}

	public function testExtended(): string {
		$v = phpversion();

		if(\Core\version_compare($v, '8.0.0', '>=')) {
			// Core Plus is untested on 8.0
			return self::STATUS_WARNING;
		}
		elseif(\Core\version_compare($v, '7.4.0', '>=')) {
			return self::STATUS_PASSED;
		}
		elseif(\Core\version_compare($v, '7.0.0', '>=')) {
			return self::STATUS_WARNING;
		}
		else {
			return self::STATUS_FAILED;
		}
	}

	public function getDetails(): string {
		$status = $this->testExtended();

		if($status === self::STATUS_PASSED) {
			return 'Server is running PHP ' . phpversion();
		}
		elseif($status === self::STATUS_WARNING) {
			return 'Server is running PHP ' . phpversion() . ', Core Plus has been tested with PHP 7.4, proceed with caution.';
		}
		else {
			return 'Core Plus recommends PHP 7.4 and requires at least PHP 7.2.';
		}
	}

	public function getFix(): string {
		return '';
	}
}