<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Parser\JPEGExif\Tags;

/**
 * Description of Flash
 *
 * @author charlie
 */
class FlashTag extends ExifFieldTag {
	protected function _parse(array &$exif) {
		
		if($this->raw === 0){
			// No flash fired, no need to say none.
			$this->formatted = '';
			return;
		}
		
		/*
		Flash:

		bits: 7   6   5   4   3   2   1   0
		dec:  128,64, 32, 16, 8,  4,  2,  1

		 */
		$flashes = [];

		/*
		Values for bit 0 indicating whether the flash fired.
		0 = Flash did not fire
		1 = Flash fired
		 */
		if(($this->raw & 1) === 0){
			$flashes[] = 'No Flash';
		}
		else{
			$flashes[] = 'Flash fired';
		}

		/*
		Values for bits 1 and 2 indicating the status of returned light.
		00 = No strobe return detection function
		01 = reserved
		10 = Strobe return light not detected
		11 = Strobe return light detected
		 */
		switch(($this->raw & 6)){
			case 4:
				$flashes[] = 'Strobe return light not detected';
				break;
			case 6:
				$flashes[] = 'Strobe return light detected';
				break;
		}

		/*
		Values for bits 3 and 4 indicating the camera's flash mode.
		00 = unknown
		01 = Compulsory flash firing
		10 = Compulsory flash suppression
		11 = Auto mode
		 */
		switch(($this->raw & 24)){
			case 8:
				$flashes[] = 'Compulsory flash mode';
				break;
			case 16:
				$flashes[] = 'Compulsory flash suppression';
				break;
			case 24:
				$flashes[] = 'Auto mode';
				break;
		}

		/*
		Values for bit 5 indicating the presence of a flash function.
		0 = Flash function present
		1 = No flash function
		 */
		if(($this->raw & 32) === 32){
			// No flash supported on this device, wipe out all previous information!
			$flashes = array('No flash supported');
		}

		/*
		Values for bit 6 indicating the camera's red-eye mode.
		0 = No red-eye reduction mode or unknown
		1 = Red-eye reduction supported
		 */
		if(($this->raw & 64) === 64){
			$flashes[] = 'Red eye reduction mode';
		}

		// I can now merge them back together.
		$this->formatted = implode(', ', $flashes);
	}
}
