<?php


namespace Core\Search;


class SearchIndex {
	const WEIGHT_TITLE = 6;
	const WEIGHT_URL = 5;
	const WEIGHT_HEADER = 4;
	const WEIGHT_KEYWORD = 3;
	const WEIGHT_SUBHEADER = 2;
	const WEIGHT_DEFAULT = 1;

	private $_words = [];

	private $_wordCache = null;
	private $_primaryCache = null;
	private $_secondaryCache = null;

	public function addText(string $text, int $weight = self::WEIGHT_DEFAULT) {
		// Clear any cache first.
		$this->_wordCache = null;
		$this->_primaryCache = null;
		$this->_secondaryCache = null;

		if(trim($text) === '') {
			// Quick bail
			return;
		}

		// Lowercase it.
		$text = strtolower($text);

		// Convert this string to latin.
		$text = \Core\str_to_latin($text);

		// Contractions get dropped with no space.
		$text = str_replace("'", '', $text);

		// Skip punctuation.
		$text = preg_replace('/[^a-z0-9 ]/', ' ', $text);

		$parts = explode(' ', $text);

		// Strip stop words that are insignificant and store the rest.
		$stopWords = \Core\get_stop_words();

		foreach($parts as $word) {
			if($word === '') {
				continue;
			}

			if(in_array($word, $stopWords)) {
				continue;
			}

			if(isset($this->_words[ $word ])) {
				$this->_words[ $word ] += $weight;
			}
			else {
				$this->_words[ $word ] = $weight;
			}
		}
	}

	public function getString() : string {
		if($this->_wordCache === null) {
			$this->_buildDoubleMetaPhone();
		}

		return $this->_wordCache;
	}

	public function getPrimary() : string {
		if($this->_primaryCache === null) {
			$this->_buildDoubleMetaPhone();
		}

		return $this->_primaryCache;
	}

	public function getSecondary() : string {
		if($this->_secondaryCache === null) {
			$this->_buildDoubleMetaPhone();
		}

		return $this->_secondaryCache;
	}

	private function _buildDoubleMetaPhone() {
		$this->_wordCache = '';
		$this->_primaryCache = '';
		$this->_secondaryCache = '';

		foreach($this->_words as $word => $weight) {
			$it = new DoubleMetaPhone($word);

			$this->_wordCache .= str_repeat($word . ' ', $weight);

			if($it->primary) {
				$this->_primaryCache .= str_repeat($it->primary . ' ', $weight);
			}
			if($it->secondary) {
				$this->_secondaryCache .= str_repeat($it->secondary . ' ', $weight);
			}
		}
	}
}