<?php
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Utilities;

use Core\ConfigHandler;
use Core\Logger\LogSinkInterface;
use Core\Logger\LogEntry;

/**
 * Provides a facility to send log messages to the system syslog (and more importantly rsyslog)
 *
 * @author charlie
 */
class SyslogLogger implements LogSinkInterface {
	
	/**
	 * Get the default level for this sink
	 * 
	 * @return int
	 */
	public function getDefaultLevel() : int{
		$level = ConfigHandler::Get('/core/logs/syslog/level');
		
		return (int)$level;
	}
	
	/**
	 * Get the maximum level (lowest priority), that this sink supports.
	 * 
	 * @return int
	 */
	public function getMaximumLevel() : int{
		return LOG_LEVEL_INFO; // Supports info and greater; no DEBUG allowed here
	}
	
	/**
	 * Record an entry into this sink's destination.
	 * 
	 * @param \Core\Logger\LogEntry $entry
	 * 
	 * @throws \Exception
	 * 
	 */
	public function log(LogEntry $entry){
		$header = '';
		
		// Convert the level of this entry to standard priorities in syslog
		switch($entry->level){
			case LOG_LEVEL_CRITICAL:
				$level = LOG_ERR;
				break;
			case LOG_LEVEL_HIGH:
				$level = $entry->type === 'error' ? LOG_WARNING : LOG_NOTICE;
				break;
			case LOG_LEVEL_INFO:
				$level = $entry->type === 'error' ? LOG_NOTICE : LOG_INFO;
				break;
			case LOG_LEVEL_DEBUG:
				$level = LOG_DEBUG;
				break;
			case LOG_LEVEL_TRACE:
				$level = LOG_DEBUG;
				break;
			default:
				$level = LOG_INFO;
		}
		
		if(EXEC_MODE == 'WEB'){
			$header .= '[' . REMOTE_IP . '] ';
		}
		else{
			$header .= '[CLI] ';
		}

		if($entry->code){
			$header .= '(' . $entry->code . ') ';
		}

		// Open the syslog connection
		$conn = openlog(HOST, LOG_NDELAY, LOG_USER);
		if(!$conn){
			throw new \Exception('Unable to open a syslog connection');
		}
		
		// Convert the message to something that is text-only friendly.
		$lines = $this->_convertMessageToText($entry->message);
		foreach($lines as $line){
			syslog($level, $line);
		}
		closelog();
	}
	
	private function _convertMessageToText(string $message) : array {
		// Convert <br/> and <br> to newlines
		$message = str_replace('<br/>', "\n", $message);
		$message = str_replace('<br>', "\n", $message);
		
		// Remove HTML strings
		$message = preg_replace('/<[^>]*>/im', '', $message);
		
		// And convert HTML codes to their character.
		$message = html_entity_decode($message);
		
		// Return an array so the logger can iterate over each and prepend the header.
		return explode("\n", $message);
	}
}
